//
//  Scanbot Barcode Scanner SDK Flutter Plugin
//
//  Copyright (c) 2020 doo GmbH. All rights reserved.
//  https://scanbot.io/sdk
//

import UIKit
import ScanbotBarcodeScannerSDK
import Flutter

class BarcodeScannerPresenter: NSObject {
    private var resultHandler: FlutterResult?
    private weak var presenter: UIViewController?
    private var shouldWaitForImage = false
    private var barcodeResults: [SBSDKBarcodeScannerResult] = []

    func present(on presenter: UIViewController,
                 with configuration: SBSDKUIBarcodeScannerConfiguration,
                 barcodeTypes: [SBSDKBarcodeType]?,
                 barcodeDocumentTypes: [SBSDKBarcodeDocumentType]?,
                 result: @escaping FlutterResult) {
        self.resultHandler = result
        self.presenter = presenter
        
        self.shouldWaitForImage = configuration.behaviorConfiguration.barcodeImageGenerationType != .none
           
        var controller: SBSDKUIBarcodeScannerViewController?
        if let documentTypes = barcodeDocumentTypes, !documentTypes.isEmpty {
            controller = SBSDKUIBarcodeScannerViewController.createNew(withAcceptedMachineCodeTypes: barcodeTypes,
                                                                        withAcceptedDocumentTypes: barcodeDocumentTypes,
                                                                        configuration: configuration,
                                                                        andDelegate: self)
        } else {
            controller = SBSDKUIBarcodeScannerViewController.createNew(withAcceptedMachineCodeTypes: barcodeTypes,
                                                                        configuration: configuration,
                                                                        andDelegate: self)
        }

        guard let controller = controller else {
            return
        }
        controller.modalPresentationStyle = .fullScreen
        presenter.present(controller, animated: true, completion: nil)
    }
    
    private func processResult(barcodeResults: [SBSDKBarcodeScannerResult], imageURL: URL?) {
        let barcodeList = barcodeResults.map({$0.encodedDictionary})
        let result: [String: Any?] = ["barcodeImageURI": imageURL?.absoluteString, "barcodeItems": barcodeList]
        self.resultHandler?(result.encodedString)
        self.resultHandler = nil
    }
}

extension BarcodeScannerPresenter: SBSDKUIBarcodeScannerViewControllerDelegate {
    func qrBarcodeDetectionViewController(_ viewController: SBSDKUIBarcodeScannerViewController,
                                          didDetect barcodeResults: [SBSDKBarcodeScannerResult]) {
        viewController.isRecognitionEnabled = false
        if self.shouldWaitForImage {
            self.barcodeResults = barcodeResults
            return
        }
        
        self.presenter?.dismiss(animated: true, completion: {
            self.processResult(barcodeResults: barcodeResults, imageURL: nil)
        })
    }
    
    func qrBarcodeDetectionViewController(_ viewController: SBSDKUIBarcodeScannerViewController,
                                          didCaptureBarcodeImage barcodeImage: UIImage?,
                                          imageURL: URL?) {
        self.presenter?.dismiss(animated: true, completion: {
            self.processResult(barcodeResults: self.barcodeResults, imageURL: imageURL)
            self.barcodeResults = []
            self.shouldWaitForImage = false
        })
    }
    
    func qrBarcodeDetectionViewControllerDidCancel(_ viewController: SBSDKUIBarcodeScannerViewController) {
        self.resultHandler = nil
        self.presenter = nil
    }
    
    func qrBarcodeDetectionViewControllerDidTimeout(_ viewController: SBSDKUIBarcodeScannerViewController) {
        self.resultHandler = nil
        self.presenter = nil
    }
}
