import 'package:barcode_scanner/barcode_parsers/medical_certificate.dart';
import 'package:barcode_scanner/barcode_parsers/sepa.dart';
import 'package:barcode_scanner/barcode_parsers/vcard.dart';

import 'aamva.dart';
import 'boarding_pass.dart';
import 'de_medical_plan.dart';
import 'id_card_pdf_417.dart';

abstract class BarcodeFormattedResult {
  String? documentFormat;

  /// Indicates whether document was successfully parsed */
  bool parsedSuccessful = false;

  BarcodeFormattedResult(this.documentFormat);

  factory BarcodeFormattedResult.fromJson(Map<String, dynamic> json) =>
      documentFormatFromJson(json);

  Map<String, dynamic> toJson() => <String, dynamic>{};
}

class UnknownBarcodeFormat extends BarcodeFormattedResult {
  UnknownBarcodeFormat(String documentFormat) : super(documentFormat);
}

BarcodeFormattedResult documentFormatFromJson(Map<String, dynamic> json) {
  var format = json['documentFormat'];
  switch (format) {
    case AAMVADocument.DOCUMENT_FORMAT:
      return AAMVADocument.fromJson(json);
    case BoardingPassDocument.DOCUMENT_FORMAT:
      return BoardingPassDocument.fromJson(json);
    case DEMedicalPlanDocument.DOCUMENT_FORMAT:
      return DEMedicalPlanDocument.fromJson(json);
    case MedicalCertificateDocument.DOCUMENT_FORMAT:
      return MedicalCertificateDocument.fromJson(json);
    case IDCardPDF417Document.DOCUMENT_FORMAT:
      return IDCardPDF417Document.fromJson(json);
    case SEPADocument.DOCUMENT_FORMAT:
      return SEPADocument.fromJson(json);
    case VCardDocument.DOCUMENT_FORMAT:
      return VCardDocument.fromJson(json);
    default:
      return UnknownBarcodeFormat('UNKNOWN');
  }
}
