import 'dart:convert';

import 'package:barcode_scanner/barcode_scanning_data.dart';
import 'package:barcode_scanner/scanbot_sdk_models.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';

import 'live_detection.dart';

class BarcodeCameraLiveDetector extends LiveDetector {
  Function(BarcodeScanningResult result) barcodeListener;
  Function(SdkLicenseException exception) errorListener;

  BarcodeCameraLiveDetector(
      {required this.barcodeListener, required this.errorListener});

  Future<dynamic> _resultHandler(MethodCall methodCall) async {
    Map<String, dynamic> args = jsonDecode(methodCall.arguments);
    switch (methodCall.method) {
      case 'onBarcodeDetected':
        if (!enabled) {
          return;
        }
        try {
          var scanningResult = BarcodeScanningResult.fromJson(args);
          barcodeListener(scanningResult);
        } catch (e) {
          Logger.root.severe(e);
        }
        return;
      case 'onLicenseError':
        errorListener.call(SdkLicenseException.fromJson(args));
        return;
      default:
    }
  }

  @override
  void registerResultHandler() {
    detectorChannel.setMethodCallHandler(_resultHandler);
  }
}
